"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

const createAdapter = (imports, implementation, options) => {
  const {
    withPerformanceTracer,
    wrapConnect,
    wrapDisconnect,
    wrapDispose,
    wrapOperation
  } = imports;
  const {
    connect,
    disconnect,
    subscribe,
    unsubscribe,
    resubscribe,
    deleteProperty,
    getProperty,
    listProperties,
    setProperty,
    resume,
    pause,
    reset,
    status,
    build,
    defineTable,
    findOne,
    find,
    count,
    insert,
    update,
    delete: del,
    ...restApi
  } = implementation;

  if (Object.keys(restApi).length > 0) {
    throw new Error(`Read-model adapter implementation should not provide extra methods: ${JSON.stringify(Object.keys(restApi))}`);
  }

  const storeApi = {
    defineTable,
    findOne,
    find,
    count,
    insert,
    update,
    delete: del
  };
  const adapterOperations = {
    subscribe,
    unsubscribe,
    resubscribe,
    deleteProperty,
    getProperty,
    listProperties,
    setProperty,
    resume,
    pause,
    reset,
    status,
    build
  };

  for (const key of Object.keys(storeApi)) {
    if (typeof storeApi[key] !== 'function') {
      throw new Error(`Store API method ${key} should be function`);
    }
  }

  for (const key of Object.keys(adapterOperations)) {
    if (typeof adapterOperations[key] !== 'function') {
      throw new Error(`Adapter operation method ${key} should be function`);
    }
  }

  const {
    performanceTracer,
    ...adapterOptions
  } = options;
  const pool = {
    commonAdapterPool: {
      performanceTracer
    },
    adapterPoolMap: new Map(),
    withPerformanceTracer,
    performanceTracer
  };
  const adapter = {
    connect: wrapConnect(pool, connect, storeApi, adapterOptions),
    disconnect: wrapDisconnect(pool, disconnect),
    dispose: wrapDispose(pool, disconnect),
    subscribe: wrapOperation(pool, 'subscribe', subscribe),
    unsubscribe: wrapOperation(pool, 'unsubscribe', unsubscribe),
    resubscribe: wrapOperation(pool, 'resubscribe', resubscribe),
    deleteProperty: wrapOperation(pool, 'deleteProperty', deleteProperty),
    getProperty: wrapOperation(pool, 'getProperty', getProperty),
    listProperties: wrapOperation(pool, 'listProperties', listProperties),
    setProperty: wrapOperation(pool, 'setProperty', setProperty),
    resume: wrapOperation(pool, 'resume', resume),
    pause: wrapOperation(pool, 'pause', pause),
    reset: wrapOperation(pool, 'reset', reset),
    status: wrapOperation(pool, 'status', status),
    build: wrapOperation(pool, 'build', build)
  };
  return Object.freeze(adapter);
};

var _default = createAdapter;
exports.default = _default;
//# sourceMappingURL=create-adapter.js.map